import json
from pathlib import Path
from typing import Dict, Union
from spacy.language import Language
from spacy.tokens import Doc

class LinkOverRider:
    def __init__(self,path,case_sensitive:bool):
        self._path = path
        self._case_sensitive = case_sensitive
    def initialize(self, get_examples=None, nlp=None):
        self.from_disk(self._path)
    def to_disk(self, path, exclude=tuple()):
        if not path.exists():
            path.mkdir()
        with open(path,"w+") as f:
            json.dump(self._unlowered_dictionary,f)
    def from_disk(self, path, exclude=tuple()):
        with open(path) as f:
            self._original_dictionary = json.load(f)
        if self._case_sensitive:
            self._overrides_dict = self._original_dictionary
        else:
            self._overrides_dict = {key.lower():value for key,value in self._original_dictionary.items()}
        return self
    def __call__(self, doc:Doc):
        overridden_ents = []
        for ent in doc.ents:
            # Overwrite the link there's an entry in the data
            if self._case_sensitive:
                ent_text = ent.text
            else:
                ent_text = ent.text.lower()
            if ent_text in self._overrides_dict:
                ent.kb_id_ = self._overrides_dict[ent_text]
            overridden_ents.append(ent)
        doc.set_ents(overridden_ents)
        return doc
    
@Language.factory(
    name="trend_literal_link_overrider",
    default_config={"case_sensitive": False},
)    
def make_ent_link_overrider(
    nlp: Language, name: str, overrides_path:Union[str,Path],case_sensitive:bool
    
):
    return LinkOverRider(overrides_path,case_sensitive)

